/*
   Hoarescope - Use Hoare logic for program correctness proofs
   Copyright (C) 2004 Markus Triska

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA

   Contact me at triska@gmx.at
*/


#include <stdio.h>
#include <stdlib.h>


#include <iostream>
#include <fstream>

using namespace std;


extern FILE *yyin;
extern int yyparse(void);
extern char *yytext;

ofstream texout("prooftree.tex");

int yyerror(char *s)
{
	printf("Error: %s near: %s\n", s, yytext);
	exit(1);
}


int main(int argc, char *argv[])
{

	cout << "I am Hoarescope version " << VERSION << endl << endl;
	if (!texout) {
		cout << "Could not open TeX output file 'prooftree.tex'" << endl;
	} else {
		texout << "\\documentclass{article}" << endl;
		texout << "\\usepackage{proof}" << endl;
		texout << "\\usepackage{bbm}" << endl;
		texout << "\\def\\stmtwhile{{\\rm while\\; }}" << endl;
		texout << "\\def\\stmtdo{{\\rm \\;do\\; }}" << endl;
		texout << "\\def\\stmtbegin{{\\rm begin\\; }}" << endl;
		texout << "\\def\\stmtend{{\\rm \\;end }}" << endl;
		texout << "\\def\\stmtif{{\\rm if\\; }}" << endl;
		texout << "\\def\\stmtthen{{\\rm \\;then\\; }}" << endl;
		texout << "\\def\\stmtelse{{\\rm \\;else\\;}}" << endl;
		texout << "\\newcommand{\\dotminus}{\\stackrel{\\textstyle .}{\\raisebox{0ex}[.6ex]{$-$}}}" << endl;
		texout << "\\newcommand{\\subst}[2]{[\\mbox{\\scriptsize$\\begin{array}{@{}c@{}}#2\\\\#1\\end{array}$}]}" << endl;


		texout << "\\begin{document}" << endl;
	}

	if (argc >= 2) {
		yyin = fopen(argv[1], "r");
		if (!yyin) {
			cout << "Could not find file: " << argv[1] << endl;
			cout << "Reading from stdin instead." << endl;
		}
	} else {
		yyin = stdin;
	}

	yyparse();

	if (texout) texout << "\\end{document}" << endl;

	return 0;
}

